package de.ozgcloud.operator.common.elasticsearch;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import de.ozgcloud.operator.IndicesPrivilegesDataTestFactory;
import de.ozgcloud.operator.PutRoleRequestDataTestFactory;
import de.ozgcloud.operator.PutUserRequestDataTestFactory;

class ElasticsearchRemoteServiceTest {

	@Spy
	@InjectMocks
	private ElasticsearchRemoteService service;
	@Mock
	private ElasticsearchClient client;
	
	@DisplayName("Create security role")
	@Nested
	class TestCreateSecurityRole {
		
		@DisplayName("create put role request")
		@Nested
		class TestCreatePutRoleRequest {
			
			@Test
			void shouldHaveName() {
				var request = service.createPutRoleRequest(PutRoleRequestDataTestFactory.create());
				
				assertThat(request.name()).isEqualTo(PutRoleRequestDataTestFactory.NAME);
			}
			
			@DisplayName("indices privileges")
			@Nested
			class TestIndicesPrivileges {
				
				@Test
				void shouldHaveName() {
					var request = service.createPutRoleRequest(PutRoleRequestDataTestFactory.create());
					
					assertThat(request.indices().get(0).names()).containsExactly(IndicesPrivilegesDataTestFactory.NAME);
				}
				
				@Test
				void shouldHavePrivileges() {
					var request = service.createPutRoleRequest(PutRoleRequestDataTestFactory.create());
					
					assertThat(request.indices().get(0).privileges()).containsExactly(IndicesPrivilegesDataTestFactory.PRIVILEGES);
				}
			}
		}
	}
	
	@DisplayName("Create security user")
	@Nested
	class TestCreateSecurityUser {
		
		@DisplayName("create put user request")
		@Nested
		class TestCreatePutUserRequest {
			
			@Test
			void shouldHaveName() {
				var request = service.createPutUserRequest(PutUserRequestDataTestFactory.create());
				
				assertThat(request.username()).isEqualTo(PutUserRequestDataTestFactory.USERNAME);
			}
			
			@Test
			void shouldHaveRoles() {
				var request = service.createPutUserRequest(PutUserRequestDataTestFactory.create());
				
				assertThat(request.roles()).containsExactly(PutUserRequestDataTestFactory.ROLES);
			}
			
			@Test
			void shouldHavePassword() {
				var request = service.createPutUserRequest(PutUserRequestDataTestFactory.create());
				
				assertThat(request.password()).isEqualTo(PutUserRequestDataTestFactory.PASSWORD);
			}
		}
	}
}