package de.ozgcloud.operator.common.kubernetes;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import io.fabric8.kubernetes.client.KubernetesClient;
import io.fabric8.kubernetes.client.extension.ResourceAdapter;
import io.javaoperatorsdk.jenvtest.junit.EnableKubeAPIServer;

@EnableKubeAPIServer
class KubernetesRemoteServiceITCase {

	private static KubernetesClient client;
	private final KubernetesRemoteService service = new KubernetesRemoteService(client);

	@DisplayName("Get secret resource")
	@Nested
	class TestGetSecretResource {
		
		@Test
		void shouldReturnEmptyResourceIfMissing() {
			var resource = service.getSecretResource(NamespaceTestFactory.NAMESPACE, SecretTestFactory.NAME);
			
			assertThat(resource.get()).isNull();
		}
		
		@Test
		void shouldReturnSecretResourceIfExists() {
			createNamespace();
			createSecret();
			
			var resource = service.getSecretResource(NamespaceTestFactory.NAMESPACE, SecretTestFactory.NAME);
			
			assertThat(resource.get()).isNotNull();
		}
		
		private void createNamespace() {
			var resource = client.namespaces().withName(NamespaceTestFactory.NAMESPACE);
			var adapter = new ResourceAdapter<>(resource);
			adapter.create(NamespaceTestFactory.create());
		}
		
		private void createSecret() {
			var resource = client.secrets().inNamespace(NamespaceTestFactory.NAMESPACE).withName(SecretTestFactory.NAME);
			var adapter = new ResourceAdapter<>(resource);
			adapter.create(SecretTestFactory.create());
		}
	}
}