/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import java.util.Optional;

import org.keycloak.representations.idm.RealmRepresentation;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.keycloak.KeycloakGenericRemoteService;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class KeycloakRealmService {

	private final KeycloakRealmRemoteService remoteService;

	private final KeycloakRealmMapper mapper;

	private final KeycloakGenericRemoteService keycloakGenericRemoteService;

	public void createRealm(OzgCloudKeycloakRealmSpec realm, String realmName) {
		Optional.of(realm)
				.map(mapper::map)
				.map(realmRepresentation -> addRealmName(realmRepresentation, realmName))
				.filter(realmRepresentation -> !keycloakGenericRemoteService.realmExists(realmName))
				.ifPresent(remoteService::createRealm);
	}

	RealmRepresentation addRealmName(RealmRepresentation realm, String realmName) {
		realm.setRealm(realmName);
		return realm;
	}

	public void deleteRealm(String realmName) {
		remoteService.deleteRealm(realmName);
	}
}
