package de.ozgcloud.operator.keycloak.user;

import java.time.Duration;
import java.util.Optional;

import de.ozgcloud.operator.keycloak.OzgCloudCustomResourceStatus;
import io.javaoperatorsdk.operator.api.reconciler.UpdateControl;

class UserUpdateControlBuilder {

	private OzgCloudKeycloakUser resource;

	private OzgCloudCustomResourceStatus status;
	private Optional<String> message = Optional.empty();

	private boolean reschedule = false;
	private Duration scheduleDuration;

	public UserUpdateControlBuilder(OzgCloudKeycloakUser resource) {
		this.resource = resource;
	}

	public static UserUpdateControlBuilder fromResource(OzgCloudKeycloakUser resource) {
		return new UserUpdateControlBuilder(resource);
	}

	public UserUpdateControlBuilder withStatus(OzgCloudCustomResourceStatus status) {
		this.status = status;
		return this;
	}

	public UserUpdateControlBuilder withMessage(String message) {
		this.message = Optional.ofNullable(message);
		return this;
	}

	public UserUpdateControlBuilder withReschedule(Duration duration) {
		this.reschedule = true;
		this.scheduleDuration = duration;
		return this;
	}

	public UpdateControl<OzgCloudKeycloakUser> build() {
		resource.setStatus(buildOzgCloudKeycloakUserStatus());

		return buildUpdateControl();
	}

	private OzgCloudKeycloakUserStatus buildOzgCloudKeycloakUserStatus() {
		var userStatus = OzgCloudKeycloakUserStatus.builder().status(status);
		message.ifPresent(userStatus::message);

		return userStatus.build();
	}

	private UpdateControl<OzgCloudKeycloakUser> buildUpdateControl() {
		if (reschedule) {
			return UpdateControl.updateStatus(resource).rescheduleAfter(scheduleDuration);
		}
		return UpdateControl.updateStatus(resource);
	}
}
